from fastapi import APIRouter, HTTPException
import requests
import logging

from starlette.status import (
    HTTP_501_NOT_IMPLEMENTED,
    HTTP_503_SERVICE_UNAVAILABLE
)

import conf

router = APIRouter()
@router.get("/health")
async def health():
    """
    ## Health check endpoint, which depends on OPA being available and healthy.
    This API does not require any headers or authentication.

    ### Possible http return status codes:
    * 200 - no error
    * 501 - not implemented
    * 503 - service not available
    """
    try:
        url = conf.OPA_HEALTH_API
    except NameError:
       logging.error(f"conf.OPA_URL undefined")
       raise HTTPException(status_code=HTTP_501_NOT_IMPLEMENTED, detail="NotHealthy - OPA configuration issue")
    except AttributeError:
       logging.error(f"conf.OPA_URL undefined")
       raise HTTPException(status_code=HTTP_501_NOT_IMPLEMENTED, detail="NotHealthy - OPA configuration issue")

    try:
        r = requests.get(url, timeout=10)
        r.raise_for_status()

    except requests.exceptions.HTTPError as e:
        logging.error(f"Http Error to OPA endpoint: {url}: {e}")
        raise HTTPException(status_code=HTTP_503_SERVICE_UNAVAILABLE, detail="NotHealthy - OPA HTTPError")
    except requests.exceptions.ConnectionError as e:
        logging.error(f"Error Connecting to OPA endpoint: {url}: {e}")
        raise HTTPException(status_code=HTTP_503_SERVICE_UNAVAILABLE, detail="NotHealthy - OPA ConnectionError")
    except requests.exceptions.Timeout as e:
        logging.error(f"Timeout Error to OPA endpoint: {url}: {e}")
        raise HTTPException(status_code=HTTP_503_SERVICE_UNAVAILABLE, detail="NotHealthy - OPA Timeout")
    except requests.exceptions.RequestException as e:
        logging.error(f"Connection error to OPA endpoint: {url}: {e}")
        raise HTTPException(status_code=HTTP_503_SERVICE_UNAVAILABLE, detail="NotHealthy - OPA Error")
    return {'message': 'Healthy'}
