import fastapi
import requests
import logging
import conf
from models.info import InfoOut, ServiceDetail, Services
import _buildinfo as b
router = fastapi.APIRouter()

@router.get("/info", response_model=InfoOut)
async def return_version_info():
    """
    Return Service version information.
    Expected returned JSON is in "InfoOut" schema, which include Services and ServiceDetail schemas.
    """
    url = conf.OPA_CONFIG_API
    opa_version = "unknown"

    try:
        r = requests.get(url, timeout=10)
        if "result" in r.text:
            data = r.json()
            opa_version = data["result"]["labels"]["version"]
    except requests.exceptions.HTTPError as e:
        logging.warning(f"Http Error to OPA {url}: {e}")
    except requests.exceptions.ConnectionError as e:
        logging.warning(f"Error Connecting to OPA {url}: {e}")
    except requests.exceptions.Timeout as e:
        logging.warning(f"Timeout Error to OPA {url}: {e}")
    except requests.exceptions.RequestException as e:
        logging.warning(f"Connection error to OPA {url}: {e}")

    svcdetail = ServiceDetail(version=opa_version)
    opa = Services(opa=svcdetail)
    info = InfoOut(
        version = b.version,
        artifactId = b.artifactId,
        name = b.name,
        groupId = b.groupId,
        buildTime = b.buildTime,
        branch = b.branch,
        commitId = b.commitId,
        commitMessage = b.commitMessage,
        connectedOuterServices=opa
    )
    return info