import logging
from fastapi import Header
from starlette_context import context, request_cycle_context
from uuid_extensions import uuid7str
from conf import GENERATE_CORRELATION_ID
import coloredlogs

#class CorrelationIDLoggerAdapter(logging.LoggerAdapter):
#    def __init__(self, logger, prefix):
#        super(CorrelationIDLoggerAdapter, self).__init__(logger, {})
#        self.prefix = prefix
#
#    def process(self, msg, kwargs):
#        return '[%s] %s' % (self.prefix, msg), kwargs

class DefaultExtrasAdapter(logging.LoggerAdapter):
    def __init__(self, logger, extra):
        super(DefaultExtrasAdapter, self).__init__(logger, extra)
        field_styles = coloredlogs.DEFAULT_FIELD_STYLES
        field_styles['correlation_id'] = {'color': 'magenta'}
        coloredlogs.install(logger=logger, fmt='%(asctime)s %(hostname)s logger=%(name)s loglevel=%(levelname)-6s correlation-id=%(correlation_id)s %(funcName)s() L%(lineno)-4d %(message)s   call_trace=%(pathname)s L%(lineno)-4d')

    def process(self, msg, kwargs):
        # Speed gain if no extras are present
        if "extra" in kwargs:
            copy = dict(self.extra).copy()
            copy.update(kwargs["extra"])
            kwargs["extra"] = copy
        else:
            kwargs["extra"] = self.extra
        return msg, kwargs

async def policy_context_dependency(correlation_id: str = Header(None), user_agent: str = Header(None)):
    # When used a Depends(), this fucntion get the `Correlation-ID` header,
    # which will be documented as a required header by FastAPI.
    # use `correlation_id: str = Header(None)` for an optional header.
    # (correlation_id = Header(...) for required header

    if not correlation_id and GENERATE_CORRELATION_ID:
        correlation_id = uuid7str()

    data = {"correlation_id": correlation_id,
            "user_agent": user_agent
           }
    with request_cycle_context(data):
        # yield allows it to pass along to the rest of the request
        yield
