# entitlement.py
#from asyncio.log import logger
import logging
from email import header
from fastapi import HTTPException
import requests
from starlette.status import HTTP_401_UNAUTHORIZED, HTTP_403_FORBIDDEN
from starlette_context import context
import json
import conf
import correlation

_logger = logging.getLogger(__name__)
logger = correlation.DefaultExtrasAdapter(_logger, {"correlation_id": "None"})

class EntitlementService:
    def request_groups(headers, timeout=10):
        logger = correlation.DefaultExtrasAdapter(_logger, {"correlation_id": context["correlation_id"]})
        logger.debug(f"{conf.ENTITLEMENTS_GROUPS_API} headers: {headers}")
        response = requests.get(conf.ENTITLEMENTS_GROUPS_API, headers = headers, timeout=timeout)
        if not response.ok:
            logger.warning(f"{conf.ENTITLEMENTS_GROUPS_API} {response.text} {response.status_code}")
            message = ""
            if "message" in response.text:
                message = json.loads(response.text)["message"]
            raise HTTPException(
                status_code=HTTP_401_UNAUTHORIZED,
                detail=f"Authentication was unsuccessful. {message} {response.status_code}",
                headers={"WWW-Authenticate": "Bearer"},
            )

        jdata = response.json()
        if "groups" in jdata:
            return jdata['groups']

        # this should never happen
        raise HTTPException(
            status_code=HTTP_401_UNAUTHORIZED,
            detail=f"Authentication was unsuccessful. Groups undefined",
            headers={"WWW-Authenticate": "Bearer"},
        )
