from typing import List, Optional, Literal
from pydantic import BaseModel
import json

"""
{
  "query": "data.search.allow == true",
  "input": {
      "groups": ["sample-group-1"],
      "operation": "view"],
    },
  },
  "unknowns": ["input.record"]
}

# a more realistic example:
    {
        "query": "data.osdu.partition[\"osdu\"].search2.allow == true",
        "input": {
            "operation": "view",
            "groups": ["AAA", "BBB"]
        },
        "unknowns": ["input.record"]
    }
"""

class TranslateBaseModel(BaseModel):
    def __hash__(self):  # make hashable BaseModel subclass
        return hash((type(self),) + tuple(self.__dict__.values()))

    def tojson(self):
        return json.dumps(self, default=lambda o: o.__dict__, 
            sort_keys=True, indent=4)

class Input(TranslateBaseModel):
    operation: Literal['view', 'create', 'update', 'delete', 'purge'] # required
    groups: list[str]                   # required
    xuserid: Optional[str] = ""         # not currently used, possibly in the future
    token: Optional[str]   = ""         # not currently used, possibly in the future
    datapartitionid: Optional[str] = "" # not currently used, possibly in the future

class TranslateItem(TranslateBaseModel):
    query: str
    input: Input
    unknowns: list[str]
