# test_integration_040_delete.py
import pytest
# requires pytest_dependency to be installed
import unittest
import responses
from fastapi.testclient import TestClient
import requests
import re
import logging
import sys
import os
import base64
import json
import time
from pathlib import Path

# so the directly in which you run pytest or unittest doesn't matter as much
sys.path.append(os.path.abspath('..'))
sys.path.append('tests')

# local
from main import app
from auth import auth
import conf
import testlib

# override dependency injection for authentication to entitlement service
from override_depends import override_require_authorized_user, set_authorize_session, ADMIN_ONLY_SVC, USER_AND_ADMIN_SVC, USER_ONLY_SVC, OTHER_ONLY_SVC

TEST_DATA_DIR = Path(__file__).resolve().parent / 'templates'

client = TestClient(app)

@pytest.mark.dependency(name="require_token")
def test_require_token(token):
    assert token is not None, "No token provided on command line"

@pytest.mark.dependency(depends=["require_token"])
def test_delete_osdu_partition_policies_service_url(token, data_partition, bundle_pause, service_url):
    """
    Test delete /policies/osdu/partition/<data_partition>/<policy>
    """
    #pytest.skip("Skipping diagnostic test")
    num_tests = 0
    p = os.path.dirname(os.path.abspath(__file__))
    template_datadir = os.path.join(p, '..', 'templates')
    for filename in sorted(os.listdir(template_datadir)):
        filetype='.rego'
        if filename.endswith(filetype):
            id = f"osdu/partition/{data_partition}/{filename}"
            url = conf.SERVICE_BASE_PATH + "/policies/" + id
            r = requests.delete(service_url + url,
                headers={'Authorization': 'Bearer ' + token,
                'data-partition-id': data_partition})
            assert r.status_code == 200, f"delete of {filename} {url}"
            assert "result" in r.text, f"result not expected {r.text}"
            num_tests = num_tests + 1
    assert num_tests >= 7, "At least 7 polices were tested"
    #time.sleep(int(bundle_pause))
