import os
import tarfile
from azure.storage.blob import BlobServiceClient
from azure.storage.blob import ContentSettings
from Helper import Helper
from pip._internal.utils._log import getLogger
from jinja2 import Environment, FileSystemLoader, select_autoescape

logger = getLogger(__name__)

class BootstrapDataPartitionBundles:

    def __init__(self, dp_id):
        logger.info("Initializing AzureBlobFileUploader")
        # Initialize the connection to Azure storage account
        helper = Helper()
        self.blob_service_client = BlobServiceClient.from_connection_string(
            helper.get_storage_connection_string())
        self.dp_id = dp_id

    def create_and_upload_dp_bundles(self):
        dp_id = self.dp_id
        tar_name = "bundle-{dp}.tar.gz".format(dp=dp_id)
        env = Environment(
            loader=FileSystemLoader('deployment/scripts/azure/templates/'),
            autoescape=select_autoescape()
        )
        dataauthz_template = env.get_template("dataauthz_template.rego")
        manifest_template = env.get_template("manifest_template.manifest")
        dataauthz_render = dataauthz_template.render(dp_id=dp_id)
        manifest_render = manifest_template.render(dp_id=dp_id)
        with open(f"dataauthz.rego","w") as f1:
            f1.write(dataauthz_render)
        with open(f".manifest", "w") as f2:
            f2.write(manifest_render)
        with tarfile.open(tar_name, "w:gz") as tar_handle:
            tar_handle.add(os.path.abspath("dataauthz.rego"), arcname="dataauthz.rego")
            tar_handle.add(os.path.abspath(".manifest"), arcname=".manifest")

        # Create blob with same name as local file name
        blob_client = self.blob_service_client.get_blob_client(container=Helper.CONTAINER_NAME,
                                                               blob=tar_name)
        # Create blob on storage
        # Overwrite if it already exists!
        image_content_setting = ContentSettings(content_type='application/x-gtar')
        logger.info("Uploading file - %s",Helper.BUNDLE_FILE_NAME)
        with open(Helper.BUNDLE_FILE_NAME, "rb") as data:
            try:
                blob_client.upload_blob(data, overwrite=True, content_settings=image_content_setting)
            except Exception as e:
                logger.error("Upload to Bundle Server failed %s", e)
                raise RuntimeError(e)



# Initialize class and upload bundles
if __name__ == '__main__':
    bootstrap = BootstrapDataPartitionBundles(os.environ.get('DATA_PARTITION_ID'))
    bootstrap.create_and_upload_dp_bundles()
