#!/usr/bin/env bash

set -ex

create_bundles() {

    echo "Archive bundle of policies"
    tar -czf bundle.tar.gz --directory='/opt/deployment/default-policies' --exclude='./bootstrap_sequence.json' . --verbose
    mkdir --parents /opt/policies ; mv bundle.tar.gz "$_"
    python3 /opt/devops/gcp/bootstrap-osdu-module/DataPartitionBundles.py
    mv /opt/bundle-"${DATA_PARTITION}".tar.gz /opt/policies
    echo "Archive is ready"

}

bootstrap_gcs() {

    echo "Push archive to GCS bucket"
    gsutil rsync /opt/policies gs://"${POLICY_BUCKET}"/
    echo "Bootsrap finished successfully"

}

bootstrap_minio() {

    echo "Installing Minio Client (mc) tool"
    apk add wget
    wget https://dl.min.io/client/mc/release/linux-amd64/mc
    chmod +x mc && mv mc /usr/bin/mc
    echo "mc tool installed successfully, configuring"
    mc alias set minio "${MINIO_HOST}":"${MINIO_PORT}" "${MINIO_ACCESS_KEY}" "${MINIO_SECRET_KEY}"
    echo "Push archive to Minio bucket"
    mc mirror --overwrite /opt/policies minio/"${POLICY_BUCKET}"
    echo "Bootsrap finished successfully"

}

if [ "${ONPREM_ENABLED}" == "true" ]
then
  source ./validate-env.sh "DATA_PARTITION"
  source ./validate-env.sh "POLICY_BUCKET"
  source ./validate-env.sh "MINIO_HOST"
  source ./validate-env.sh "MINIO_ACCESS_KEY"
  source ./validate-env.sh "MINIO_SECRET_KEY"
  source ./validate-env.sh "MINIO_PORT"
  create_bundles
  bootstrap_minio
else
  source ./validate-env.sh "DATA_PARTITION"
  source ./validate-env.sh "POLICY_BUCKET"
  create_bundles
  bootstrap_gcs
fi

touch /tmp/bootstrap_ready
