# Policy Service - Admin CLI
The Admin CLI is an easy to use full featured CLI.
This AdminCLI was new in OSDU Milestone 14. It was ready for production use then.
Now with M15 it's even better.  As always, please report any issues.

##### For help:
* General help: `pol.py --help`
* Individual command help is also available, for example: `pol.py ls --help`

##### The main policy commands are:
* `add` for adding or updating policies. This is particular useful for automation and loading policies into OSDU partitions,
* `eval` for evaluating policies,
* `ls` for listing and retrieving policies and
* `rm` for deleting policies
* `health` - retrieves health status of policy service

##### Additional Policy Developer Utils
* `compile`      Use OPA's Compile API to partially evaluate a query.
* `config`       Diagnostic config on Policy Service.
* `diff`         Compare two policies, show the delta in a context diff format.
* `opa-add`      Add or update a policy directly in OPA ✨ for LOCAL testing/development
* `opa-rm`       delete a policy directly from OPA. 🔥 for LOCAL testing/development
* `translate`    For helping testing translate which is used by search service via os-core-common

##### Additional Utils/commands include:
* `groups` - Showing groups related to your auth context,
* `info` - Retrieves info from services,
* `legal-tags` - Get legal tags from legal tag service,
* `search` -  Search Utility
* `storage` - Storage and Dataset record retrieval utility

You will need to set the following environmental variables or provide details on command line:
* `TOKEN` or `--token`
* `DATA_PARTITION` or `--data-partition-id`
* `BASE_URL` or `--base-url` or `--url` or `--host`
See [setenv.sh](setenv.sh) for an example of these.
Please note: *command-line options will override environment variables.*

Command line completion is available:
* `--install-completion`
* `--show-completition`

##### Development / Testing Notes:
Individual services can be optionally redirected for development, testing or custom environments:
* `POLICY_URL` or `--policy-url`
* `ENTITLEMENTS_URL` or `--ent-url`
* `STORAGE_URL` or `--storage-url`
* `SEARCH_URL` or `--search-url`
* `LEGAL_URL` or `--legal-url`
* `DATASET_URL` or `--dataset-url`
These are not required.  See [setenv.sh](setenv.sh) for an example of these.
Please note: *command-line options will override environment variables.*
Errors during collecting phase are generally issues with TOKEN.

##### Built-in Template Engine
Add, eval and translate support a templating engine that makes it super easy to automate. Hopefully you find this helpful.

When using --template option the strings in our file input will be replaced, for example:
* `${data_partition}` will be replaced by the data partition id you are using
* `${name}` will be replaced by the policy id you selected in the command
See the individual command's help for template support details.

In future releases a compiled version of the CLI may be made available.

##### Output options for commands:
Some commands support multiple output options for example `pol.py ls --output=<>` or `pol.py search --output=<>
* fancy (default with colors and formatting)
* simple
* excel (supported only on search currently)
* tree (tree output of policies)

If the policy admincli detects there isn't a tty simple should automatically be selected to make automation easier.

In addition many commands support feature to get raw json return from OSDU service using the `--raw` option.

##### Searching:
Search uses standard Regular Expressions (shortened as regex or regexp), sometimes referred to as rational expressions - a sequence of characters that specifies a search pattern in text. This is extremely powerful search facility.

Most letters and characters will simply match themselves. For example `test` will match the string `test` exactly. There are exceptions to this rule; some characters are special metacharacters, and don’t match themselves. Instead, they signal that some out-of-the-ordinary thing should be matched, or they affect other portions of the regex by repeating them or changing their meaning. 
These include: `. ^ $ * + ? { } [ ] \ | ( )`

The `ls` command supports two kinds of searching for policies:
* `--search` which takes a REGEX search string. This will search in both policy name and the stored policy rego.
* `--name` which takes a REGEX search string will only search in policy names.

##### Force Option:
Some commands `add`, `eval` and `translate` will be default ask for comfirmation when changing data or using templating. You can bypass this by using option `--force` which is great for automation.

**The policy service team would appreciate any feedback and feature requests on the AdminCLI.**

##### Building executables:
`make build` should build executable for your architecture in dist directory. Be sure to use python 3.9.x

##### Building container and running container:
* `make docker_build`
* `make run` - you will still need to set the environment variables
* `make run_local` - you will still need to set the environment variables but this will override POLICY_URL to point to your local machine via docker host.
