import os
import requests
import subprocess
import tempfile

class OpaCheck:
    fpath = None
    def __init__(self):
        self.tempdir = tempfile.mkdtemp()

    def createfile(self, rego: str, filename: str):
        self.fpath = os.path.join(self.tempdir, filename)
        f = open(self.fpath, "w")
        f.write(rego)
        f.close()

    def opa_check_file(self):
        try:
            result = subprocess.run(['opa', 'check', self.fpath], capture_output=True, text=True)
        except FileNotFoundError as err:
            os.remove(self.fpath)
            return(-1,"OPA Binary not found")
        os.remove(self.fpath)
        return(result.returncode, result.stderr)

    def check(self, rego: str, filename: str):
        self.createfile(rego, filename)
        return self.opa_check_file()

    def delete(self):
        os.rmdir(self.tempdir)

def put_opa_policy_direct(policy_id, data, base_url, timeout=20):
    """
    Put a policy in OPA directly - for development/LOCAL use
    """
    url = base_url + '/v1/policies/' + policy_id

    try:
        rsp = requests.put(url, data=data, headers={'Content-Type': 'application/octet-stream'}, timeout=timeout)
    except requests.exceptions.RequestException as e: 
        raise SystemExit(e)
    
    return rsp

def delete_opa_policy_direct(policy_id, base_url, timeout=20):
    """
    Put a policy in OPA directly - for development/LOCAL use
    """
    url = base_url + '/v1/policies/' + policy_id

    try:
        rsp = requests.delete(url, headers={'Content-Type': 'application/octet-stream'}, timeout=timeout)
    except requests.exceptions.RequestException as e: 
        raise SystemExit(e)
    
    return rsp