#!/usr/bin/env bash
# USAGE:
# -c CLOUD_PROVIDER
while getopts c: flag
do
    case "${flag}" in
        c) CLOUD_PROVIDER=${OPTARG};;
    esac
done

python3 --version
python3 -m venv env
source env/bin/activate
python3 -m pip install --upgrade pip
python3 -m pip install -r requirements.txt
python3 -m pip install -r requirements-dev.txt

# AWS
if [ ${CLOUD_PROVIDER} == "aws" ]; then
    export POLICY_URL=$AWS_BASE_URL
    export TOKEN=$(python3 ../../app/tests/aws/aws_jwt_client.py)
    export ENTITLEMENTS_URL="${AWS_BASE_URL}/api/entitlements/v2/groups"
    export LEGAL_URL="${AWS_BASE_URL}/api/legal/v1/legaltags"
elif [ ${CLOUD_PROVIDER} == "local-gcp" ]; then
    export DATA_PARTITION=osdu
    export TOKEN="$(gcloud auth print-access-token)"
    export POLICY_URL="http://localhost:8080"
    export ENTITLEMENTS_URL="https://policy-dev.osdu.lol/api/entitlements/v2/groups"
    export LEGAL_URL="https://policy-dev.osdu.lol/api/legal/v1/legaltags"
else
  echo "Not supported cloud provider ${CLOUD_PROVIDER}"
  exit 1
fi

if [ -z $TOKEN ]; then
    echo "[ERROR] Not able to get token (TOKEN)"
    exit 1
fi

echo DATA_PARTITION $DATA_PARTITION
echo CLOUD_PROVIDER: $CLOUD_PROVIDER
echo POLICY_URL: $POLICY_URL
echo ENTITLEMENTS_URL: $ENTITLEMENTS_URL
echo LEGAL_URL: $LEGAL_URL
python3 -m pytest -v 
