from rich.table import Table
from rich.console import Console
import xlsxwriter
import os, sys, subprocess
import typer
import random

console = Console()
error_console = Console(stderr=True, style="bold red")

def display_search_results_excel(data,
    show_kind = False,
    excelfile = "search.xlsx", 
    worksheetname = "Search Results"):

    # Workbook() takes one, non-optional, argument   
    # which is the filename that we want to create. 
    workbook = xlsxwriter.Workbook(excelfile) 
        
    # The workbook object is then used to add new   
    # worksheet via the add_worksheet() method.  
    worksheet = workbook.add_worksheet(worksheetname) 
        
    # Create a new Format object to formats cells 
    # in worksheets using add_format() method. 
    # here we create bold format object. 
    bold = workbook.add_format({'bold': 1}) 
    
    headings = ['ID', "Kind", "Authority", "Source", "Type", "Create Time", "Create User"]

    # Write a row of data starting from 'A1' with bold format
    worksheet.write_row('A1', headings, bold)

    # Start from the first cell below the headers.
    row = 1
    col = 0
    for item in data["results"]:
        worksheet.write_row(row=row, col=0, data=(
            item['id'], item["kind"], item["authority"], item["source"], item["type"], item["createTime"], item["createUser"]
            ))
        row += 1

    worksheet.set_column(0, 0, 50)  # Column  A width set to 20.
    worksheet.set_column(1, 1, 30)  # Columns B width set to 30.
    worksheet.set_column(2, 3, 8)   # Columns C-D width set to 8.
    worksheet.set_column(4, 4, 20)  # Columns width set to 25.
    worksheet.set_column(5, 6, 25)  # Columns width set to 25.

    # Finally, close the Excel file
    workbook.close()
    console.print(f"Search results ({row-1} records) saved as '{excelfile}'")

def display_search_results_simple(data):
    for item in data["results"]:
        created = item["createTime"]

        print(item['id'], item["kind"], item["authority"], item["source"], item["type"], created)

def display_search_results_idonly(data):
    for item in data["results"]:
        print(item['id'])

def display_search_results_idonly_random(data):
    lucky = random.choice(data["results"])
    print(lucky['id'])

def display_search_results_fancy(data, show_kind=False):
    count=len(data["results"])
    if not count:
        return
    table = Table(title=f"Search Results ({count})")
    table.add_column("ID", justify="full", style="cyan", no_wrap=True, min_width=25)
    if show_kind:
        table.add_column("Kind", justify="full", style="cyan", no_wrap=True, min_width=25)
    table.add_column("Authority", justify="center", style="green", no_wrap=True, max_width=10)
    table.add_column("Source", justify="center", style="green", no_wrap=True, max_width=8)
    table.add_column("Type", justify="center", style="green", no_wrap=True, min_width=15)
    table.add_column("Created", justify="center", style="green", no_wrap=True, max_width=11)

    for item in data["results"]:
        created = item["createTime"]
        if show_kind:
            table.add_row(item['id'], item["kind"], item["authority"], item["source"], item["type"], created)
        else:
            table.add_row(item['id'], item["authority"], item["source"], item["type"], created)
    console.print(table)

def open_file(filename):
    if sys.platform == "win32":
        os.startfile(filename)
    else:
        opener = "open" if sys.platform == "darwin" else "xdg-open"
        subprocess.call([opener, filename])

def search_order_callback(ctx: typer.Context, value: str):
    if value != "ASC" and value != 'DESC':
        raise typer.BadParameter("Only ASC or DESC is allowed")
    return value