import sys
import os
import pytest
from typer.testing import CliRunner
sys.path.append(os.path.abspath('..'))
from pol import cli

runner = CliRunner()

@pytest.mark.skipif(sys.version_info == (3, 9), reason="requires python3.9")

@pytest.mark.parametrize("rego_path,policy_name",
    [("tests/example", "example"),
     ("tests/example", "example2"),
     ("../../app/tests/templates/search2","search2")])
def test_cli_policy_add_policy(rego_path, policy_name):
    result = runner.invoke(cli, ["add", "-f", f"{rego_path}.rego", "-t", policy_name, "--force"])
    assert result.exit_code == 0

def test_cli_policy_ls(data_partition):
    result = runner.invoke(cli, ["ls"])
    assert result.exit_code == 0
    assert "osdu/instance/legal.rego" in result.stdout
    assert "osdu/instance/entitlements.rego" in result.stdout
    assert "osdu/instance/dataauthz.rego" in result.stdout
    assert f"osdu/partition/{data_partition}/dataauthz.rego" in result.stdout

def test_cli_policy_ls_raw(data_partition):
    result = runner.invoke(cli, ["ls", "--raw"])
    assert result.exit_code == 0
    assert "osdu/instance/legal.rego" in result.stdout
    assert "osdu/instance/entitlements.rego" in result.stdout
    assert "osdu/instance/dataauthz.rego" in result.stdout
    assert f"osdu/partition/{data_partition}/dataauthz.rego" in result.stdout

def test_cli_policy_ls_output_tree():
    result = runner.invoke(cli, ["ls", "--output=tree"])
    assert result.exit_code == 0

def test_cli_policy_ls_output_fancy(data_partition):
    result = runner.invoke(cli, ["ls", "--output=fancy"])
    assert result.exit_code == 0
    assert "osdu/instance/legal.rego" in result.stdout
    assert "osdu/instance/entitlements.rego" in result.stdout
    assert "osdu/instance/dataauthz.rego" in result.stdout
    assert f"osdu/partition/{data_partition}/dataauthz.rego" in result.stdout

def test_cli_policy_ls_output_simple(data_partition):
    result = runner.invoke(cli, ["ls", "--output=simple"])
    assert result.exit_code == 0
    assert "osdu/instance/legal.rego" in result.stdout
    assert "osdu/instance/entitlements.rego" in result.stdout
    assert "osdu/instance/dataauthz.rego" in result.stdout
    assert f"osdu/partition/{data_partition}/dataauthz.rego" in result.stdout

def test_cli_policy_ls_dataauthz(data_partition):
    result = runner.invoke(cli, ["ls", "dataauthz"])
    assert f"package osdu.partition[\"{data_partition}\"].dataauthz" in result.stdout
    assert result.exit_code == 0

def test_cli_policy_ls_dataauthz_download(data_partition):
    result = runner.invoke(cli, ["ls", "dataauthz", "--download"])
    assert f"saved as" in result.stdout
    assert result.exit_code == 0
    assert os.path.exists("dataauthz.rego"), f"dataauthz.rego does not exist"
    os.remove("dataauthz.rego")

def test_cli_policy_ls_dataauthz_search_download_raw(data_partition):
    result = runner.invoke(cli, ["ls", "dataauthz", "search", "--raw", "--download"])
    assert f"raw saved as" in result.stdout
    assert result.exit_code == 0
    assert os.path.exists("dataauthz.json"), f"dataauthz.json does not exist"
    assert os.path.exists("search.json"), f"dataauthz.json does not exist"
    os.remove("dataauthz.json")
    os.remove("search.json")

def test_cli_policy_ls_stdin_dataauthz(data_partition):
    result = runner.invoke(cli, ["ls", "-"], input="dataauthz\n")
    assert result.exit_code == 0
    assert f"package osdu.partition[\"{data_partition}\"].dataauthz" in result.stdout

def test_cli_policy_ls_more_than_1_legal_dataauthz(data_partition):
    result = runner.invoke(cli, ["ls", "osdu/instance/legal.rego", "dataauthz"])
    assert 'package osdu.instance.legal' in result.stdout
    assert f"package osdu.partition[\"{data_partition}\"].dataauthz" in result.stdout
    assert result.exit_code == 0

def test_cli_policy_ls_dataauthz_rego(data_partition):
    result = runner.invoke(cli, ["ls", "dataauthz.rego"])
    assert f"package osdu.partition[\"{data_partition}\"].dataauthz" in result.stdout
    assert result.exit_code == 0

def test_cli_policy_ls_osdu_partition_osdu_dataauthz(data_partition):
    result = runner.invoke(cli, ["ls", "osdu/partition/" + data_partition +"/dataauthz"])
    assert f"package osdu.partition[\"{data_partition}\"].dataauthz" in result.stdout
    assert result.exit_code == 0

def test_cli_policy_ls_osdu_partition_osdu_dataauthz_rego(data_partition):
    result = runner.invoke(cli, ["ls", "osdu/partition/" + data_partition +"/dataauthz.rego"])
    assert f"package osdu.partition[\"{data_partition}\"].dataauthz" in result.stdout
    assert result.exit_code == 0

def test_cli_policy_ls_osdu_partition_osdu_search2(data_partition):
    result = runner.invoke(cli, ["ls", "osdu/partition/" + data_partition +"/search2.rego"])
    if result.exit_code:
        pytest.xfail(f"Policy search2 not available yet")
    assert f"package osdu.partition[\"{data_partition}\"].search2" in result.stdout
    assert result.exit_code == 0

def test_cli_policy_ls_osdu_partition_osdu_search2_quiet(data_partition):
    result = runner.invoke(cli, ["ls", "osdu/partition/" + data_partition +"/search2.rego"])
    if result.exit_code:
        pytest.xfail(f"Policy search2 not available yet")
    result = runner.invoke(cli, ["ls", "osdu/partition/" + data_partition +"/search2.rego", "--quiet"])
    assert result.exit_code == 0

def test_cli_policy_diff(data_partition):
    result = runner.invoke(cli, ["ls", "osdu/partition/" + data_partition +"/search2.rego", "--quiet"])
    if result.exit_code:
        pytest.xfail(f"Policy search2 not available yet")
    result = runner.invoke(cli, ["diff", "search", "search2"])
    assert result.exit_code == 0

def test_cli_policy_diff_n(data_partition):
    result = runner.invoke(cli, ["ls", "osdu/partition/" + data_partition +"/search2.rego", "--quiet"])
    if result.exit_code:
        pytest.xfail(f"Policy search2 not available yet")
    result = runner.invoke(cli, ["diff", "search", "search2", "-n", "0"])
    assert result.exit_code == 0

def test_cli_policy_health():
    result = runner.invoke(cli, ["health"])
    assert result.exit_code == 0
    assert "Healthy" in result.stdout

def test_cli_info_default():
    result = runner.invoke(cli, ["info"])
    assert result.exit_code == 0
    assert "version" in result.stdout

@pytest.mark.parametrize("service", [("policy"), ("dataset"), ("storage"), ("search"), ("entitlement"), ("legal")])
def test_cli_info(service):
    result = runner.invoke(cli, ["info", "-s", service])
    assert result.exit_code == 0
    assert "version" in result.stdout

def test_cli_group():
    result = runner.invoke(cli, ["groups"])
    assert result.exit_code == 0
    assert "service.policy." in result.stdout

def test_cli_group_all():
    result = runner.invoke(cli, ["groups", "--all"])
    assert result.exit_code == 0

def test_cli_group_domain():
    result = runner.invoke(cli, ["groups", "--domain"])
    assert result.exit_code == 0

def test_cli_group_domain_suffix():
    result = runner.invoke(cli, ["groups", "--domain-suffix"])
    assert result.exit_code == 0

def test_cli_legaltags():
    result = runner.invoke(cli, ["legal-tags"])
    assert result.exit_code == 0

def test_cli_legaltags_raw():
    result = runner.invoke(cli, ["legal-tags", "--raw"])
    assert result.exit_code == 0

def test_cli_legaltags_limit1():
    result = runner.invoke(cli, ["legal-tags", "--limit=1"])
    assert result.exit_code == 0

def test_cli_legaltags_random():
    result = runner.invoke(cli, ["legal-tags", "--random"])
    assert result.exit_code == 0

def test_cli_legaltags_lookup():
    result = runner.invoke(cli, ["legal-tags", "--random"])
    legal_tag = result.stdout.strip()
    result = runner.invoke(cli, ["legal-tags", legal_tag])
    assert result.exit_code == 0

def test_cli_legaltags_add():
    result = runner.invoke(cli, ["add-legal-tag", "-f", "tests/legaltag_create.json"])
    if result.exit_code == 2:
        pytest.xfail("legal tag already exists")
    assert result.exit_code == 0

def test_cli_legaltags_add_extensions():
    result = runner.invoke(cli, ["add-legal-tag", "-f", "tests/legaltag_create_extension.json"])
    if result.exit_code == 2:
        pytest.xfail("legal tag already exists")
    assert result.exit_code == 0

def test_cli_legaltags_update():
    result = runner.invoke(cli, ["add-legal-tag", "-f", "tests/legaltag_update.json", "--update"])
    assert result.exit_code == 0

# eval
def test_cli_policy_eval():
    legal_result = runner.invoke(cli, ["legal-tags", "--random"])
    legal_tag = legal_result.stdout.strip()
    result = runner.invoke(cli, ["ls", "search2", "--quiet"])
    if result.exit_code:
        pytest.xfail(f"Policy search2 not available yet")
    result = runner.invoke(cli, ["eval", "--output=simple", "-f", "tests/evaluate_query.json", "-t", "search2", f"--legal-tag={legal_tag}", "--force"])
    assert result.exit_code == 0, f"expected 0 exit status. stdout: {result.stdout}"
    assert "{}" in result.stdout 

# translate
def test_cli_policy_translate():
    result = runner.invoke(cli, ["ls", "search2", "--quiet"])
    if result.exit_code:
        pytest.xfail(f"Policy search2 not available yet")
    result = runner.invoke(cli, ["translate", "-f", "tests/translate_data.json", "-t", "search2", "--force"])
    assert result.exit_code == 0
    assert "query" in result.stdout
    assert "bool" in result.stdout
    assert "should" in result.stdout
    assert "filter" in result.stdout

def test_cli_show_completion():
    result = runner.invoke(cli, ["--show-completion"])
    if result.exit_code == 2:
        pytest.skip("Not supported env")
    assert result.exit_code == 0