import sys
import os
import pytest
from typer.testing import CliRunner

from tests.conftest import NUM_SEARCH_STORAGE_DATASET_TESTS, RANDOM_SEARCH_LMIT

sys.path.append(os.path.abspath('..'))
from pol import cli

runner = CliRunner()

"""
Test AdminCLI Search and Storage/Dataset commands and workflows
While this absolutely tests AdminCLI this is really geared towards 
preship validation testing.
"""
@pytest.mark.skipif(sys.version_info == (3, 9), reason="requires python3.9")

def test_cli_search():
    result = runner.invoke(cli, ["search"])
    assert result.exit_code == 0

def test_cli_search_raw():
    result = runner.invoke(cli, ["search", "--raw"])
    assert "createTime" in result.stdout
    assert result.exit_code == 0

def test_cli_search_limit1():
    result = runner.invoke(cli, ["search", "--limit=1"])
    assert "dataset" or "osdu" or "opendes" in result.stdout
    assert result.exit_code == 0

def test_cli_search_limit2():
    result = runner.invoke(cli, ["search", "--limit", "2"])
    assert "dataset" or "osdu" or "opendes" in result.stdout
    assert result.exit_code == 0

def test_cli_search_output_simple():
    result = runner.invoke(cli, ["search", "--output=simple"])
    assert "dataset" or "osdu" or "opendes" in result.stdout
    assert result.exit_code == 0

def test_cli_search_output_fancy():
    result = runner.invoke(cli, ["search", "--output=fancy"])
    assert "dataset" or "osdu" or "opendes" in result.stdout
    assert result.exit_code == 0

def test_cli_search_output_excel():
    result = runner.invoke(cli, ["search", "--output=excel"])
    assert result.exit_code == 0

def test_cli_search_id_only():
    result = runner.invoke(cli, ["search", "--id-only"])
    assert "dataset" or "osdu" or "opendes" in result.stdout
    assert result.exit_code == 0

def test_cli_search_random_id():
    result = runner.invoke(cli, ["search", "--random-id"])
    assert "dataset" or "osdu" or "opendes" in result.stdout
    assert result.exit_code == 0

def test_cli_search_query_well():
    result = runner.invoke(cli, ["search", "--query", "well"])
    # Will exit 2 if there isn't well data loaded
    if result.exit_code == 2:
        pytest.xfail(f"No well search data")
    assert result.exit_code == 0, f"unexpected return status {result.exit_code} {result.stdout}"

def test_cli_search_kind():
    result = runner.invoke(cli, ["search", "--kind", '*:*:*:*'])
    assert "dataset" or "osdu" or "opendes" in result.stdout
    assert result.exit_code == 0

def test_cli_search_storage_workflow_thefirst():
    # Get an ID from search
    result = runner.invoke(cli, ["search", "--limit=1", "--id-only"])
    assert result.exit_code == 0
    id = result.stdout.strip()
    check_storage_record(id)

def test_cli_search_doesnotexist():
    result = runner.invoke(cli, ["search", "doesnotexist"])
    assert "No results found" in result.stdout
    assert result.exit_code == 2

def test_cli_storage_doesnotexist():
    result = runner.invoke(cli, ["storage", "doesnotexist"])
    assert "Not a valid record id" in result.stdout
    assert result.exit_code == 1

def test_cli_search_storage_workflow_a_random_outof_100():
    # select a random record out of 100
    result = runner.invoke(cli, ["search", "--limit=100", "--random-id"])
    assert result.exit_code == 0
    id = result.stdout.strip()
    check_storage_record(id)

def test_cli_search_storage_workflow_a_random_outof_1000():
    # select a random record out of 100
    result = runner.invoke(cli, ["search", "--limit=1000", "--random-id"])
    assert result.exit_code == 0
    id = result.stdout.strip()
    check_storage_record(id)

def test_cli_search_storage_workflow_first25():
    # select a random record out of 100
    result = runner.invoke(cli, ["search", "--limit=25", "--id-only"])
    assert result.exit_code == 0
    for id in result.stdout.splitlines():
        print(id)
        check_storage_record(id.strip())

def check_storage_record(id):
    assert "dataset" or "osdu" or "opendes" in id
    result = runner.invoke(cli, ["storage", id])
    if "Record not found" in result.stdout:
        pytest.xfail(f"Record {id} not found in storage")
    assert "data" in result.stdout
    assert "createTime" in result.stdout
    assert "createUser" in result.stdout
    if "DatasetProperties" not in result.stdout:
        pytest.xfail(f"No dataset info for {id} in storage, likely No DMS handler for kindSubType")
    assert "DatasetProperties" in result.stdout
    assert "FileSourceInfo" in result.stdout
    assert result.exit_code == 0

def test_cli_search_storage_dataset_workflow_thefirst():
    # Get an ID from search
    result = runner.invoke(cli, ["search", "--limit=1", "--id-only"])
    id = result.stdout.strip()
    check_storage_dataset_record(id)

def test_cli_search_storage_dataset_workflow_a_random_outof_100():
    # Get an ID from search
    result = runner.invoke(cli, ["search", "--limit=100", "--random-id"])
    id = result.stdout.strip()
    check_storage_dataset_record(id)

def test_cli_search_storage_dataset_workflow_first25():
    # select a random record out of 100
    result = runner.invoke(cli, ["search", "--limit=25", "--id-only"])
    assert result.exit_code == 0
    for id in result.stdout.splitlines():
        print(id)
        check_storage_dataset_record(id.strip())

def check_storage_dataset_record(id):
    assert "dataset" or "osdu" or "opendes" in id
    result = runner.invoke(cli, ["storage", id, "--dataset"])
    assert "datasetRegistries" in result.stdout
    assert result.exit_code == 0

def test_cli_search_storage_dataset_download_workflow_thefirst():
    # Get an ID from search
    result = runner.invoke(cli, ["search", "--limit=1", "--id-only"])
    assert result.exit_code == 0
    id = result.stdout.strip()
    check_storage_get_record_download(id)

def get_random_storage_ids(n, limit=100):
    idlist = []
    for x in range(n):
        idlist.append(get_random_storage_id(limit=limit))
    print(idlist)
    return idlist

def get_random_storage_id(limit=10):
    result = runner.invoke(cli, ["search", f"--limit={limit}", "--random-id"])
    assert result.exit_code == 0
    id = result.stdout.strip()
    return id

@pytest.mark.parametrize("id", get_random_storage_ids(NUM_SEARCH_STORAGE_DATASET_TESTS, limit=RANDOM_SEARCH_LMIT))
def test_cli_random_search_storage_dataset_download(id):
    check_storage_record(id)
    check_storage_dataset_record(id)
    check_storage_get_record_download(id)

def test_cli_search_storage_dataset_download_workflow_a_random_outof_100():
    id = get_random_storage_id(limit=100)
    check_storage_get_record_download(id)

def test_cli_search_storage_dataset_download_workflow_first15():
    # Get an ID from search
    result = runner.invoke(cli, ["search", "--limit=15", "--id-only"])
    assert result.exit_code == 0
    for id in result.stdout.splitlines():
        print(id)
        check_storage_get_record_download(id.strip())

def check_storage_get_record_download(id):
    assert "dataset" or "osdu" or "opendes" in id
    result = runner.invoke(cli, ["storage", id, "--get"])
    if "No DMS handler" in result.stdout:
        pytest.xfail(f"Download not supported for id {id}")
    if "Collection Path is missing in the record metadata" in result.stdout:
        pytest.xfail(f"Bad record metadata {id}")
    if "FileSourceInfo Name Path missing" in result.stdout:
        pytest.xfail(f"Bad record metadata {id}")
    if "Dataset Metadata does not contain dataset" in result.stdout:
        pytest.xfail(f"Bad record metadata {id}")
    assert result.exit_code == 0, f"Expected storage id {id} --get to have details on dataset stdout:{result.stdout}"
    result = runner.invoke(cli, ["storage", id, "--download", "--raw"])
    assert result.exit_code == 0
    result = runner.invoke(cli, ["storage", id, "--download"])
    assert result.exit_code == 0
    assert "Downloaded" in result.stdout
    filename = result.stdout.split(' as ')[1].replace("'",'')
    filename = filename.strip()
    print(f"stdout: {result.stdout}")
    filename = os.path.basename(filename)
    assert os.path.exists(filename), f"downloaded file {filename} does not exist {result.stdout}"
    os.remove(filename)