import time
import os
import sys
import logging
from locust import HttpUser, task, between, tag, events
from string import Template
sys.path.append('../app')
from conf import SERVICE_BASE_PATH

class PolicyServiceUser(HttpUser):
    #wait_time = between(1, 5)
    host = os.environ.get('SERVICE_URL')
    search_enabled = False

    def on_start(self):
        """
        Run on startup
        """
        token = os.environ.get('TOKEN')
        self.data_partition = os.environ.get('DATA_PARTITION')

        # setup headers for all API calls
        headers = {
            'Accept': 'application/json',
            'Authorization': 'Bearer ' + token,
            'data-partition-id': self.data_partition,
        }
        self.client.headers = headers

        # load rendered translate json at startup
        self.translate_data = self.load_translate_data(
            policy_id = "search2",
            template_file = "../frontend/admincli/tests/translate_data.json"
        )

        # Let's make sure search2.rego exists, otherwise translate will fail
        url = SERVICE_BASE_PATH + "/policies/osdu/partition/" + self.data_partition + "/" + "search2.rego"
        with self.client.get(url, catch_response=True) as r:
            if r.status_code == 200:
                self.search_enabled = True
    
    def load_translate_data(self, policy_id, template_file):
        """
        Load template data...
        In template replace:
        ${name} with policy_id
        ${data_partition} with data_partition
        """
        f = open(template_file, "r")
        data = f.read()
        f.close()
        template_data = Template(data)
        data = template_data.substitute(
            {'data_partition': self.data_partition,
            'name': policy_id
        })
        return data

    @tag('get_policy')
    # weight this task higher
    @task(20)
    def get_policy(self):
        """
        Get a single policy, dataauthz
        """
        policy_id = "dataauthz.rego"
        data_partition = os.environ.get('DATA_PARTITION')
        url = SERVICE_BASE_PATH + "/policies/osdu/partition/" + data_partition + "/" + policy_id
        with self.client.get(url, catch_response=True) as r:
            if r.status_code == 200:
                #print(r.json())
                r.success()
            else:
                r.failure(f"Failed: {r.status_code} {r.text}")

    @tag('translate')
    # weight this task higher
    @task(20)
    def translate(self):
        """
        translate

        """
        if self.search_enabled:
            url = SERVICE_BASE_PATH + "/translate"
            with self.client.post(url, data=self.translate_data, catch_response=True) as r:
                if r.status_code == 200:
                    #print(r.json())
                    r.success()
                else:
                    r.failure(f"Failed: {r.status_code} {r.text}")

    @tag('health')
    @task
    def health(self):
        """
        Get a health for service, which checks health of OPA
        """
        self.client.get(SERVICE_BASE_PATH + "/health")

@events.quitting.add_listener
def _(environment, **kw):
    if environment.stats.total.fail_ratio > 0.05:
        logging.error("Test failed due to failure ratio > 5%")
        environment.process_exit_code = 1
    elif environment.stats.total.avg_response_time > 400:
        logging.error("Test failed due to average response time ratio > 400 ms")
        environment.process_exit_code = 1
    elif environment.stats.total.get_response_time_percentile(0.95) > 800:
        logging.error("Test failed due to 95th percentile response time > 800 ms")
        environment.process_exit_code = 1
    else:
        environment.process_exit_code = 0
