#!/usr/bin/env bash
# USAGE:
# -c CLOUD_PROVIDER
while getopts c: flag
do
    case "${flag}" in
        c) CLOUD_PROVIDER=${OPTARG};;
    esac
done

python3 --version
python3 -m venv env
source env/bin/activate
python3 -m pip install --upgrade pip
python3 -m pip install -r requirements.txt

# AWS
if [ ${CLOUD_PROVIDER} == "aws" ]; then
    export SERVICE_URL=$AWS_BASE_URL
    export TOKEN=$(python3 ../app/tests/aws/aws_jwt_client.py)
    sleep 150
elif [ ${CLOUD_PROVIDER} == "azure" ]; then
    python3 -m pip install -r requirements.txt
    export TOKEN=$(curl -ks -XPOST "https://login.microsoftonline.com/${AZURE_TENANT_ID}/oauth2/token" -d "grant_type=client_credentials&client_id=${AZURE_PRINCIPAL_ID}&client_secret=${AZURE_PRINCIPAL_SECRET}&resource=${AZURE_APP_ID}" | jq --raw-output '.access_token')
    export SERVICE_URL=https://${AZURE_DNS_NAME}
    sleep 30
elif [ ${CLOUD_PROVIDER} == "local-gcp" ]; then
    export DATA_PARTITION=osdu
    export TOKEN="$(gcloud auth print-access-token)"
    export SERVICE_URL="http://localhost:8080"
else
  echo "Not supported cloud provider ${CLOUD_PROVIDER}"
  exit 1
fi

if [ -z $TOKEN ]; then
    echo "[ERROR] Not able to get token (TOKEN)"
    exit 1
fi

echo DATA_PARTITION $DATA_PARTITION
echo CLOUD_PROVIDER: $CLOUD_PROVIDER
echo SERVICE_URL=$SERVICE_URL
locust --headless --users 10 --spawn-rate 1 --run-time 10m -H $SERVICE_URL --only-summary --csv=stats_${CLOUD_PROVIDER}_
